﻿/*---------------------------------------------------------------------------
* 							Dr. Fritz Faulhaber GmbH & Co.KG
*----------------------------------------------------------------------------
* Class CMomanLibSample.
* Provides wrapper functions for selected MomanLib functions,  
* defines the enum values used from Momancmd.h file.
*----------------------------------------------------------------------------
*/
using System;
using System.Runtime.InteropServices;

namespace DemoMomanLib
{
    public delegate void tdmmProtDataCallback();
    public delegate void tdmmProtTraceValuesCallback(int nodeNr, UInt32[] value, int timecode);
    public enum eMomanprot
    {
        eMomanprot_ok_bootup = 2,
        eMomanprot_ok_async = 1,
        eMomanprot_ok = 0,
        eMomanprot_error = -1,
        eMomanprot_error_timeout = -2,
        eMomanprot_error_cmd = -3,
        eMomanprot_error_emcy = -4,
        eMomanprot_error_param = -5,
        eMomanprot_error_accessdenied = -6,
        eMomanprot_error_init = -7,
        eMomanprot_noData = -8
    }
    public enum eMomancmd {
	    //Device Control:
	    eMomancmd_shutdown = 16,
	    eMomancmd_switchon = 17,
	    eMomancmd_disable = 18,
	    eMomancmd_quickstop = 19,
	    eMomancmd_DiOp = 20,
	    eMomancmd_EnOp = 21,
	    eMomancmd_faultreset = 22,
	    eMomancmd_MA = 23,
	    eMomancmd_MR = 24,
	    eMomancmd_HS = 25
    }
    public enum eDecoded {
	    eDecoded_none = 0,    		/*!< none decoded */
	    eDecoded_SOBJ = 1,			/*!< SOBJ command decoded */
	    eDecoded_GOBJ = 2,			/*!< GOBJ command decoded*/
	    eDecoded_Bootup = 3,
	    eDecoded_NMT = 4,
	    eDecoded_NMTRequest = 5,
	    eDecoded_Heartbeat = 6,
	    eDecoded_Statusword = 7,
    };

    public static class MomanWrapperLib
    {
        //Used protocol Dll from communication Library:
        public const string cProtDll = "../../../Lib/Bin/Protocol/USB/CO_USB.dll";

        //Load needed functions of the communication library:
        [DllImport(cProtDll, CallingConvention=CallingConvention.StdCall)]
        public static extern eMomanprot  mmProtInitInterface(string InterfaceDll, tdmmProtDataCallback DataReceived, tdmmProtTraceValuesCallback TraceValuesReceived);
        [DllImport(cProtDll, CallingConvention=CallingConvention.StdCall)]
        public static extern void mmProtCloseInterface();
        [DllImport(cProtDll, CallingConvention=CallingConvention.StdCall)]
        public static extern eMomanprot mmProtOpenCom(int port, int channel, int baud);
        [DllImport(cProtDll, CallingConvention=CallingConvention.StdCall)]
        public static extern void mmProtCloseCom();
        [DllImport(cProtDll, CallingConvention = CallingConvention.StdCall)]
        public static extern bool mmProtSendCommand(int nodeNr, int index, int subIndex, int dataLen, int data);
        [DllImport(cProtDll, CallingConvention = CallingConvention.StdCall)]
        public static extern eMomanprot mmProtReadAnswer(out IntPtr answData, out int nodeNr, out IntPtr cmdString, out IntPtr receiveTelegram);
        [DllImport(cProtDll, CallingConvention = CallingConvention.StdCall)]
        public static extern eDecoded mmProtDecodeAnswStr([MarshalAs(UnmanagedType.LPStr)] string answStr, out Int64 value);
        [DllImport(cProtDll, CallingConvention = CallingConvention.StdCall)]
        public static extern eMomanprot mmProtGetStrObj(int nodeNr, int index, int subIndex, out IntPtr value);
        [DllImport(cProtDll, CallingConvention = CallingConvention.StdCall)]
        public static extern eMomanprot mmProtSetObj(int nodeNr, int index, int subIndex, int value, int len, out uint abortCode);
        [DllImport(cProtDll, CallingConvention = CallingConvention.StdCall)]
        public static extern string mmProtGetAbortMessage(uint abortCode);
    }

    public class CMomanLibSample
    {
        //Used interface Dll from communication Library:
        const string cIntfDll = "../../../Lib/Bin/Interface/USB/MC3USB.dll";

        bool m_isComOpen = false;
        int m_Statusword = 0;
        string m_abortMessage = "";

        public CMomanLibSample() 
        { 
            //check if we can find the dll
            if (!System.IO.File.Exists(MomanWrapperLib.cProtDll))
            {
                throw new DllNotFoundException();
            }
        }
        ~CMomanLibSample() 
        {
            if(m_isComOpen)
            {
                MomanWrapperLib.mmProtCloseCom();
                MomanWrapperLib.mmProtCloseInterface();
            }
        }

        /**
        * Initialize the communication Library
        *
        * \param[in]	SignalDataReceived      callback function to signal received data
        * \return		true = initialization succesfull
        *				false = error loading the library
        */
        public bool Init(tdmmProtDataCallback SignalDataReceived) 
        {
            if (MomanWrapperLib.mmProtInitInterface(cIntfDll, SignalDataReceived, null) != eMomanprot.eMomanprot_ok)
            {
                return false;
            }
            if (MomanWrapperLib.mmProtOpenCom(1, 0, 0) != eMomanprot.eMomanprot_ok)
            {
                return false;
            }
            m_isComOpen = true;
            return true;
        }

        /**
        * Read data of a string object in the object dictionary
        *
        * \param[in]	nodeNr		node number
        * \param[in]	index		object index
        * \param[in]	subIndex	object subindex
        * \param[out]	value		object data
        * \return		true = object successfully read
        *				false = error reading object value
        */
        public bool GetStrObj(int nodeNr, int index, int subIndex, out string value)
        {
            IntPtr answData;
            eMomanprot ret = MomanWrapperLib.mmProtGetStrObj(nodeNr, index, subIndex, out answData);
            if (ret == eMomanprot.eMomanprot_ok)
            {
                value = Marshal.PtrToStringAnsi(answData);
                return true;
            }
            else
            {
                value = "<ERROR>";
                return false;
            }
        }

        /**
        * Send a Motion Manager command
        *
        * \param[in]	nodeNr		node number
        * \param[in]	cmd			Motion Manager command
        * \return		true = command successfully sent
        *				false = error sending command
        */
        public bool SendCommand(int nodeNr, eMomancmd cmd)
        {
            return MomanWrapperLib.mmProtSendCommand(nodeNr, 0x0000, (int)cmd, 0, 0);
        }

        /**
        * Read asynchronous received data
        *
        * \param[out]	data	    received data
        * \param[out]	cmd		    command assigned to the data
        * \return		true = data received
        *				false = no data received
        */
        public bool ReadReceivedData(out string data, out string cmd)
        {
            IntPtr answData;
            IntPtr cmdString;
            IntPtr receiveTelegram;
            int nodeNr;
            eMomanprot ret = MomanWrapperLib.mmProtReadAnswer(out answData, out nodeNr, out cmdString, out receiveTelegram);
            if (ret != eMomanprot.eMomanprot_noData)
            {
                long value;
                data = Marshal.PtrToStringAnsi(answData);
                cmd = Marshal.PtrToStringAnsi(cmdString);
                if (MomanWrapperLib.mmProtDecodeAnswStr(data, out value) == eDecoded.eDecoded_Statusword)
                {
                    m_Statusword = (int)value;
                }
                return true;
            }
            else
            {
                data = "";
                cmd = "";
                return false;
            }
        }

        /**
        * Write data to an object in the object dictionary
        *
        * \param[in]	nodeNr		node number
        * \param[in]	index		object index
        * \param[in]	subIndex	object subindex
        * \param[in]	value		object data
        * \param[in]	len			len of object data
        * \return		true = object successfully written
        *				false = error writing object value
        *				    	  (an abort message is written to a member variable)
        */
        public bool SetObj(int nodeNr, int index, int subIndex, int value, int len)
        {
            uint abortCode;
            eMomanprot ret = MomanWrapperLib.mmProtSetObj(nodeNr, index, subIndex, value, len,out abortCode);
            if (ret == eMomanprot.eMomanprot_ok)
            {
                return true;
            }
            else
            {
                m_abortMessage = MomanWrapperLib.mmProtGetAbortMessage((uint)abortCode);
                return false;
            }
        }

        /**
        * Read last value of the statusword
        *
        * \return		value of statusword set in ReadReceivedData()
        */
        public int GetStatusword()
        {
            return m_Statusword;
        }

        /**
        * Read last abort message
        *
        * \return		abort message set in SetObj()
        */
        public string GetAbortMessage()
        {
            return m_abortMessage;
        }
    }
}
