# encoding: utf-8
# @Author: FAULHABERmotioncontrol
# @Date:   2021-07-01T07:59:41+02:00
# @Email:  FAULHABERMotioncontrol@users.noreply.github.com
# @Project: motioncontrollib
# @Filename: console.py
# @Last modified by:   FAULHABERmotioncontrol
# @Last modified time: 2023-05-12T12:04:35+02:00
# @License: # This module is part of motioncontrollib and is released under
# <see LICENSE file>
# @Copyright: # Copyright (C) 2020-2020 the motioncontrollib authors and contributors
# <see AUTHORS file>


"""
example for using this module in a command line and logging values.

if you want a commandline interface use click, argparse or other good libraries
"""
import logging
from collections import namedtuple
from contextlib import contextmanager

import momanlibpy

logging.basicConfig(
    format="[%(levelname)-5.5s]  %(message)s",
    level=logging.DEBUG,
    handlers=[logging.StreamHandler()],
)


@contextmanager
def protocol(
    protocol_dll=momanlibpy.DLL_PATH_CO_USB,
    interface_dll=momanlibpy.DLL_PATH_MC3USB,
):
    """function to automaticall load a device from usb

    (for can use CO_CAN and IXXAT_VCI3 or whatever adapter you have)
    """
    intf = momanlibpy.MomanIntf(interface_dll)
    interfacelist = []
    port_amount, ports, channels, deviceinfos = intf.mmIntfEnumPorts()
    interfaceType = namedtuple("Intf", ["port", "channel", "devinfo"])

    # interfacelist is now [(port, channel, devinfo), ...]
    interfacelist = [
        interfaceType(port=int(x[0]), channel=int(x[1]), devinfo=x[2])
        for x in zip(ports, channels, deviceinfos)
        if len(x[0]) > 0
    ]

    logging.info(f"intfacelist: {interfacelist}")
    if len(interfacelist) == 0:
        logging.info("no devices detected")
        exit(0)
    selected_interface = interfacelist[0]

    if intf.mmIntfIsPortAvailable(selected_interface.port, selected_interface.channel):
        protocol = momanlibpy.MomanProt(protocol_dll=protocol_dll)
        interface_result = protocol.mmProtInitInterface(interface_dll)
        if interface_result is momanlibpy.eMomanprot.eMomanprot_ok:
            logging.debug("open interface was successful")
            baudrates_list = protocol.mmProtSupportedBaudratesList()
            baudrate = baudrates_list[0] if len(baudrates_list) > 0 else 0
            open_result = protocol.mmProtOpenCom(
                selected_interface.port,
                selected_interface.channel,
                baudrate,  # 0 for usb
            )
            if open_result:
                logging.debug("open was successful")
                yield protocol
        else:
            logging.debug(f"open was not successful {interface_result}")
        try:
            # trying to close,
            # as described in EN_7000_05064.pdf 3.1.1 Synchronous communication
            protocol.mmProtCloseCom()
            protocol.mmProtCloseInterface()

        except OSError:
            pass  # we are done here anyways
    else:
        logging.debug("port not available")


def main():
    """main function for this example"""
    import time
    with protocol() as proto:
        traceTrigger = momanlibpy.STraceTrigger()  # not used
        traceSetup = momanlibpy.STraceSetup()
        traceSetup.run = True
        traceSetup.nodeNr = 1
        traceSetup.chan = 1  # Logger uses CAN-PDOs
        traceSetup.mode = 0  # Logger
        traceSetup.source[0] = 0x606400  # Trace parameter 1: actual position
        traceSetup.source[1] = 0x606C00  # Trace parameter 2: actual speed
        traceSetup.source[2] = 0  # Trace parameter 3: not used
        traceSetup.source[3] = 0  # Trace parameter 4: not used
        traceSetup.sourceLen[0] = 4
        traceSetup.sourceLen[1] = 4
        traceSetup.sourceLen[2] = 0
        traceSetup.sourceLen[3] = 0
        traceSetup.sourceType[0] = 0x481  # CobId of trace pdo for Source0
        traceSetup.sourceType[1] = 0x481  # CobId of trace pdo for Source1
        traceSetup.sourceType[2] = -1  # Deactivated
        traceSetup.sourceType[3] = -1  # Deactivated

        def cb(nodeNr, value, timecode):
            cb.values += [(nodeNr, value, timecode)]
            return True

        cb.values = []

        ret_callback = proto.mmProtSetTraceValuesCallback(cb)
        ret_setup = proto.mmProtSetupTrace(traceSetup, traceTrigger)

        logging.debug(f"setup: {ret_setup}, trace: {ret_callback}")
        for i in range(10):
            retX = proto.mmProtRequestTrace(0)
            logging.debug(retX)
            time.sleep(0.1) # is required for thread switch and time of travel
        logging.debug(f"{cb.values}") # should be [(1, [xxxx, xxxx, 0, 0], -1), ...]
        logging.info("done with tracing")


if __name__ == "__main__":
    main()
